local agent_helper = require(AGENT_PATH .. "agent_helper")

AgentAdapter = {
	is_init = false,
	is_login_verify_ing = false,
	login_callback = nil,
	force_logout = false,
	login_verify_url = "http://cls.cp01.youyannet.com/ht6/login_verify.php",
	pay_url = "http://cls.cp01.youyannet.com/ht6/pay.php",
	uid = "",
}

-- 初始化
function AgentAdapter:Init()

	if self.is_init then return end
	self.uid = string.gsub((AdapterToLua:getInstance():getDataCache("PRVE_ACCOUNT_NAME") or ""), self:GetSpid() .. "_", "")
	self.is_init = true
	AgentMs:InitLocalPush()
	PlatformBinder:JsonBind("call_agent_logout", function(arg)
		self:LogoutCallback(arg)
		return true
	end)
end

-- 平台ID
function AgentAdapter:GetSpid()
	return "ht6"
end
-- 平台帐号唯一标识
function AgentAdapter:GetOpenId()
	return self.uid
end

-- 加前辍的平台帐号，全平台唯一
function AgentAdapter:GetPlatName()
	return "ht6_".. self.uid
end

--获得游戏名字
function AgentAdapter:GetGameName()
	return Language.Common.GameName[41]
end

--获得logo资源名
function AgentAdapter:GetLogoResName()
	return "big_logo_sgqy"
end

--获得loadingbgname
function AgentAdapter:GetLoadingbgName()
	return "loading_login_sgqy_ht6"
end

function AgentAdapter:Login(callback)
	if self.is_login_verify_ing then
		SysMsgCtrl.Instance:ErrorRemind("正在登录验证中，请稍等...", true)
		return
	end
	self.login_callback = callback
	agent_helper:Login(LUA_CALLBACK(self, self.LoginCallback))--设置登录回调 

end

function AgentAdapter:OnClickBackLogin()
	local callback = function()
		AgentLoginView:OnClickBtnLogin()--重新调出登录界面
    end
    --自己按“返回”切换账号
	PlatformBinder:JsonCall("call_agent_logout", "", "", LUA_CALLBACK(self, callback))
end

function AgentAdapter:LoginCallback(result)--登录后回调

	local res = result	

	if nil == res then return end

	local user_id = res

	local verify_callback = function(url, arg, data, size)
		
		self.is_login_verify_ing = false
		if size <= 0 then
			return
		end
		
		local ret_t = cjson.decode(data)
		if nil == ret_t or 0 ~= ret_t.ret or nil == ret_t.user then
			return
		end
	
		self.uid = ret_t.user.uid
		LoginController.Instance:SetLoginUserData(self:GetPlatName(), ret_t.user)
		self.login_callback(self:GetPlatName())

		agent_helper:ShowFloatButton()
	end


	self.is_login_verify_ing = true
	local now_server_time = math.floor(GLOBAL_CONFIG.server_info.server_time + (Status.NowTime - GLOBAL_CONFIG.client_time))
	local real_url = string.format("%s?spid=%s&userid=%s", self.login_verify_url, self:GetSpid(), user_id)
	Log("log_real_url====" .. real_url)
	HttpClient:Request(real_url, "", verify_callback)

	return false
	
end

function AgentAdapter:LogoutCallback(result)--注销回调

	self.uid = ""
	if not LoginController.Instance:AgentLoginViewIsOpen() then

		ReStart()
	end
	return true
end

function AgentAdapter:UIEventCallback(event)--UI事件回调
	if "open" == event then
		--self:ShowFloatButton()
	elseif "close" == event then
		--self:ShowFloatButton()
	end
	return true
end


function AgentAdapter:Pay(role_id, role_name, amount, server_id, callback)--购买


	if self.uid == "" then
		SysMsgCtrl.Instance:ErrorRemind("uid为空，请联系客服")
		return
	end

	local payurl = self.pay_url
	local CpOrderId = server_id .. "_" .. self.uid .. "_" .. os.time()
	local param_obj = {}
	param_obj.CpOrderId = CpOrderId
	param_obj.prodName = "元宝"
	param_obj.amount = amount
	param_obj.ratio = 10
	param_obj.serverid = server_id
	param_obj.payurl = payurl
	param_obj.uid = tostring(role_id)

	--Log("param_obj:" .. cjson.encode(param_obj))

	agent_helper:Pay(cjson.encode(param_obj), nil)

end

-- 进入游戏上报
function AgentAdapter:SubmitRoleData(role_id, role_name, role_level, zone_id, zone_name)
	local info = {}
	info.roleId = tostring(role_id)
	info.area = tostring(zone_id) .. zone_name
	info.roleName = role_name
	info.guild = RoleData.Instance.role_vo.guild_name or "帮派"
	info.vip = tostring(RoleData.Instance.role_vo.vip_level or 0)
	info.level = tostring(role_level)
	info.extInfo = ""
	
	agent_helper:SubmitExtendData(cjson.encode(info))
end

--  升级上报
function AgentAdapter:ReportOnRoleLevUp(role_id, role_name, role_level, zone_id, zone_name)
	local info = {}
	info.roleId = tostring(role_id)
	info.area = tostring(zone_id) .. zone_name
	info.roleName = role_name
	info.guild = RoleData.Instance.role_vo.guild_name or "帮派"
	info.vip = tostring(RoleData.Instance.role_vo.vip_level or 0)
	info.level = tostring(role_level)
	info.exeInfo = ""
	
	agent_helper:SubmitExtendData(cjson.encode(info))
	

end

--退出
function PlatformAdapter:OpenExitDialog()

	if GLOBAL_CONFIG.package_info.version > 50000000 then--hs6小米用到

		local format = { title = "退出", message = "退出游戏?", positive = "确定", negative = "取消", }
		self:OpenAlertDialog(format, function (result) if "positive" == result then 

				AdapterToLua:endGame()

		end end)

	else
		--通知SDK退出
		agent_helper:Exit(function(result)
			if "true" == result then
				AdapterToLua:endGame()
			end
		end)
	end

end

--是否支持动态下载
function AgentAdapter:IsDynamicDownloadRes()
	return true
end

--是否关闭推送
function AgentAdapter:IsClosePush()
	return true
end