socket = require("socket")
mine = require  "mime"
cjson = require("cjson.safe")

PLATFORM = 0										-- 平台
IS_IOS_OR_ANDROID = false							-- 是否是IOS或者安卓
NOW_TIME = 1										-- 当前时间，从程序启动开始算
IS_ON_CROSSSERVER = false							-- 是否跨服中

GLOBAL_CONFIG = {}									-- 全局配置
AGENT_PATH = ""										-- 平台路径
IS_AUDIT_VERSION = false							-- 是否审核版本

function Sleep(n)
	socket.select(nil, nil, n)
end

function LUA_CALLBACK(object, func)
	return function(...)
		return func(object, ...)
	end
end

-- 重排列搜索路径
-- @is_res_priority 是否资源优先
function SortSearchPaths(is_res_priority)
	local paths = cc.FileUtils:getInstance():getSearchPaths()

	local temp_paths = {{}, {}, {}}
	local res_index = is_res_priority and 1 or 2
	local scripts_index = is_res_priority and 2 or 1

	for i, v in ipairs(paths) do
		if string.sub(v, -5) == "/res/" then
			table.insert(temp_paths[res_index], v)
		elseif string.sub(v, -9) == "/scripts/" then
			table.insert(temp_paths[scripts_index], v)
		else
			table.insert(temp_paths[3], v)
		end
	end

	paths = {}
	for _, v in ipairs(temp_paths) do
		for _, path in ipairs(v) do
			table.insert(paths, path)
		end
	end
	cc.FileUtils:getInstance():setSearchPaths(paths)
end

-- 程序入口
function Start()
	SortSearchPaths(false)

	require("cocos2d/DeprecatedEnum")
	require("cocos2d/DeprecatedClass")
	require("cocos2d/Deprecated")
	require("preload/http_client")
	PLATFORM = CCApplication:getInstance():getTargetPlatform()
	IS_IOS_OR_ANDROID = (cc.PLATFORM_OS_ANDROID == PLATFORM or cc.PLATFORM_OS_IPHONE == PLATFORM or cc.PLATFORM_OS_IPAD == PLATFORM)

	if cc.PLATFORM_OS_ANDROID == PLATFORM then
		require("platform/android/platform_adapter")
	elseif cc.PLATFORM_OS_IPHONE == PLATFORM then
		require("platform/iphone/platform_adapter")
	elseif cc.PLATFORM_OS_IPAD == PLATFORM then
		require("platform/ipad/platform_adapter")
	elseif cc.PLATFORM_OS_WINDOWS == PLATFORM then
		require("platform/windows/platform_adapter")
	elseif PLATFORM_OS_MAC == PLATFORM then
		require("platform/mac/platform_adapter")
	else
		require("platform/platform_adapter")
	end

	require("preload/global_config")
	require("preload/agent_global_config")

	-- init director
	local director = cc.Director:getInstance()
	director:setDisplayStats(false)					-- 是否显示右下状态文字
	director:setAnimationInterval(1.0 / 60)			-- 设置帧间隔

	-- init glview
	local glview = director:getOpenGLView()
	if nil == glview then
		glview = cc.GLView:createWithRect("ttaxx", cc.rect(0, 0, 1136, 640))
		director:setOpenGLView(glview)
	end

	-- 闪屏时间1.5秒
	local screen_time = 0

	local design_height = 768
	if cc.PLATFORM_OS_IPAD == PLATFORM then
		design_height = 896
	else
		local frame_size = glview:getFrameSize()
		if frame_size.height >= 768 and frame_size.width / frame_size.height <= 1.51 then
			design_height = 896
		end
	end
	glview:setDesignResolutionSize(1536, design_height, cc.ResolutionPolicy.FIXED_HEIGHT)

	local size = director:getWinSize()
	AdapterToLua:GetGameScene():SetViewRect(cc.rect(0, 0, size.width, size.height))

	AdapterToLua:getInstance():setUpdateHandler(Update)
	AdapterToLua:getInstance():setStopHandler(Stop)

	-- start prober
	require("preload/prober_ctrl")
	MainProber:Start()

	local login_times = AdapterToLua:getInstance():getDataCache("LOGIN_TIMES")
	if "" == login_times then
		login_times = 1
	else
		login_times = tostring(tonumber(login_times) + 1)
	end
	AdapterToLua:getInstance():setDataCache("LOGIN_TIMES", login_times)

	MainProber:Step(MainProber.STEP_LUA_BEG or 30, GLOBAL_CONFIG.assets_info.version)
	if MainProber.Step2 then MainProber:Step2(100, "", "") end

	function start_func()
		-- start loader
		require("preload/loader_ctrl")
		MainLoader:Start()
	end

	-- 只有第一次有闪屏
	if 1 ~= tonumber(login_times) then
		start_func()
		return
	end

	-- ios只有快用有闪屏
	local agent_id = GLOBAL_CONFIG.package_info.config.agent_id
	if "iky" ~= agent_id and (cc.PLATFORM_OS_IPHONE == PLATFORM or cc.PLATFORM_OS_IPAD == PLATFORM) then
		start_func()
		return
	end

	if "hq5" == agent_id or "hy1" == agent_id or "hy2" == agent_id  or "hy3" == agent_id or "hy4" == agent_id or "hy5" == agent_id then
		PlatformBinder:JsonCall("call_agent_init")--闪屏，要用HandleAgentInit来播放
	end

	-- 特殊闪屏。某些渠道需要单独加个闪屏
	function SpecialSplashScreen(param, color)
		local loyout_logo_bg = ccui.Layout:create()
		loyout_logo_bg:setContentSize(size)
		loyout_logo_bg:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
		loyout_logo_bg:setBackGroundColor(color or cc.c3b(0xff, 0xff, 0xff))
		AdapterToLua:GetGameScene():addChildToRenderGroup(loyout_logo_bg, GRQ_UI_UP)

		local logo = ccui.ImageView:create(param, ccui.TextureResType.localType)
		logo:setPosition(size.width / 2, size.height / 2)
		
		
		loyout_logo_bg:addChild(logo)
		local delay = cc.DelayTime:create(1.5)
		local fade_out = cc.FadeOut:create(0.3)
		local call_back = cc.CallFunc:create(function()
			loyout_logo_bg:removeFromParent()
			-- start_func()
		end)
		local action = cc.Sequence:create(delay, fade_out, call_back)
		loyout_logo_bg:runAction(action)
		
		return logo
	end

	
	local loyout_logo_bg = ccui.Layout:create()
	loyout_logo_bg:setContentSize(size)
	loyout_logo_bg:setBackGroundColorType(ccui.LayoutBackGroundColorType.solid)
	loyout_logo_bg:setBackGroundColor(cc.c3b(0xff, 0xff, 0xff))
	AdapterToLua:GetGameScene():addChildToRenderGroup(loyout_logo_bg, GRQ_UI_UP)

	local img_logo = ccui.ImageView:create("ui/login/Default_Landscape.png", ccui.TextureResType.localType)
	img_logo:setPosition(size.width / 2, size.height / 2)

	local res_landscape_path_name = "agentres/landscape.png"
	if cc.FileUtils:getInstance():isFileExist(res_landscape_path_name) then
		screen_time = 1.5
		SpecialSplashScreen(res_landscape_path_name)
	else
		local agent_id = GLOBAL_CONFIG.package_info.config.agent_id or "dev"
		local agent_login = GLOBAL_CONFIG.login[agent_id]
		if nil ~= agent_login and "" ~= agent_login then
			local is_flash_screen = agent_login.is_flash_screen and true or false
			local is_two_flash_screen = agent_login.is_two_flash_screen and true or false
			local two_flash_screen_path = agent_login.two_flash_screen_path or ""
			local is_flash_screen_big = agent_login.is_flash_screen_big and true or false
			local mini_logo_path = agent_login.mini_logo_path or ""
			local big_logo = nil

			if agent_id == "hy1" and GLOBAL_CONFIG.package_info.version > 20000000 then
				two_flash_screen_path = nil
			end

			-- 是否有闪屏
			if is_flash_screen then
				loyout_logo_bg:addChild(img_logo)
			else
				--闪屏时间为零
				screen_time = 0
			end

			if "" ~= two_flash_screen_path and nil ~= two_flash_screen_path then
				screen_time = 1.5
				big_logo = SpecialSplashScreen(two_flash_screen_path)
			end

			-- 二次闪屏处理时间3秒
			if is_two_flash_screen then
				screen_time = 3
			end

			--闪屏需要更屏幕一样大
			if is_flash_screen_big and nil ~= big_logo then
				big_logo:setScaleX(size.width / big_logo:getContentSize().width)
				big_logo:setScaleY(size.height / big_logo:getContentSize().height)
			end

			if "" ~= mini_logo_path and nil ~= mini_logo_path then
				local mini_logo = ccui.ImageView:create(mini_logo_path, ccui.TextureResType.localType)
				mini_logo:setPosition(size.width / 2, size.height / 10)
				loyout_logo_bg:addChild(mini_logo)
			end
		end
	end
	
	local delay = cc.DelayTime:create(screen_time)
	local fade_out = cc.FadeOut:create(0.3)
	local call_back = cc.CallFunc:create(function()
		loyout_logo_bg:removeFromParent()
		start_func()
	end)
	local action = cc.Sequence:create(delay, fade_out, call_back)
	loyout_logo_bg:runAction(action)

	
end

function Update(dt)
	NOW_TIME = NOW_TIME + dt

	if nil ~= HttpClient then
		HttpClient:Update(dt)
	end

	if nil ~= MainLoader then
		MainLoader:Update(dt)
	end

	if nil ~= MainProber then
		MainProber:Update(dt)
	end
end

function Stop()
	if nil ~= MainLoader then
		MainLoader:Stop()
		MainLoader = nil
	end

	if nil ~= HttpClient then
		HttpClient:Stop()
		HttpClient = nil
	end

	if nil ~= MainProber then
		MainProber:Stop()
		MainProber = nil
	end
end

-- 进入后台
function EnterBackground()
	cc.Director:getInstance():stopAnimation()
	cc.SimpleAudioEngine:getInstance():pauseMusic()

	if nil ~= MainLoader then
		MainLoader:EnterBackground()
	end
end

-- 进入前台
function EnterForeground()
	cc.Director:getInstance():startAnimation()
	cc.SimpleAudioEngine:getInstance():resumeMusic()

	if nil ~= MainLoader then
		MainLoader:EnterForeground()
	end
end

-- 网络状态改变
function NetStateChanged(net_state)
	print("NetStateChanged " .. net_state)

	if nil ~= MainLoader then
		MainLoader:NetStateChanged(net_state)
	end
	return true
end

-- 全局配置变更
function GlobalConfigChanged()
	if nil ~= MainProber then
		MainProber:GlobalConfigChanged()
	end
	
	if nil ~= MainLoader then
		MainLoader:GlobalConfigChanged()
	end
end

-- 内存警告
function MemoryWarning()
	if nil ~= ClientCmdCtrl and nil ~= ClientCmdCtrl.Instance then
		ClientCmdCtrl.Instance:ClearMemory(false)
	end

	if nil ~= MainProber then
		MainProber:Warn(MainProber.EVENT_MEMORY_WARN)
	end
end

function __G__TRACKBACK__(msg)
	local log = debug.traceback() .. "\n[LUA-ERROR] " .. msg
	if nil ~= MainProber then
		MainProber:Error(log)
	else
		print(log)
	end

	return msg
end

math.randomseed(os.time())

local status, msg = xpcall(Start, __G__TRACKBACK__)
